#!/bin/sh
# Deploy langpack-o-matic charm
# Author: Martin Pitt <martin.pitt@ubuntu.com>

set -e

MYDIR=$(dirname $(readlink -f $0))

wait_deployed() {
    echo "waiting for $1 to get deployed..."
    while true; do
        juju_status="$(juju status $1)"
        if echo "$juju_status" | grep -q 'agent-state: started' &&
           ! echo "$juju_status" | grep -q 'agent-state: pending'; then
            break
        fi
        sleep 5
    done
}

subordinate_charms() {
    if [ -n "$MOJO_PROJECT" ]; then
        echo 'deploying subordinate charms to $1'
        juju add-relation ksplice "$1"
        juju add-relation landscape-client "$1"
    fi
}

GPG_PUB_KEY=$(gpg --export -a language-packs@ubuntu.com)
GPG_SECRET_KEY=$(gpg --export-secret-keys -a language-packs@ubuntu.com)
if [ -z "$GPG_PUB_KEY" -o -z "$GPG_SECRET_KEY" ]; then
    echo "ERROR: GPG key for language-packs@ubuntu.com not available" >&2
    exit 1
fi

# create charm config

config_yaml=$(mktemp)
trap "rm $config_yaml" EXIT INT QUIT PIPE

#
# deploy subordinate charms for automatic machine management (only if
# $MOJO_PROJECT is set, i. e. for the production environment in ProdStack)
#

if [ -n "$MOJO_PROJECT" ]; then
    [ -d "$MYDIR/charms/trusty/landscape-client" ] || bzr checkout --lightweight lp:charms/trusty/landscape-client "$MYDIR/charms/trusty/landscape-client"
    [ -d "$MYDIR/charms/trusty/ksplice" ] || { echo "Please check out ksplice charm to $MYDIR/charms/trusty"; exit 1; }

    #
    # install/update basenode into charms
    #
    [ -d "$MYDIR/basenode" ] || { echo "Please check out basenode into $MYDIR"; exit 1; }
    for charmdir in $MYDIR/charms/trusty/*; do
        # ignore subordinate charms
        if grep -q 'subordinate:.*true' $charmdir/metadata.yaml; then
            continue
        fi
        echo "Installing basenode into $charmdir"
        rm -rf "$charmdir/exec.d/basenode"
        mkdir -p "$charmdir/exec.d"
        cp -r "$MYDIR/basenode" "$charmdir/exec.d"
    done

    if ! juju status | grep -q ksplice:; then
        juju deploy --repository "$MYDIR/charms" local:trusty/ksplice
        juju set ksplice accesskey=$(cat /srv/mojo/LOCAL/$MOJO_PROJECT/canonical-is-ksplice.key)
        juju set ksplice source=""
    fi
    if ! juju status | grep -q landscape-client:; then
        cat <<EOF >> "$config_yaml"
landscape-client:
  url: https://landscape.is.canonical.com/message-system
  ping-url: http://landscape.is.canonical.com/ping
  account-name: standalone
  registration-key: $(cat /srv/mojo/LOCAL/$MOJO_PROJECT/canonical-is-landscape.key)
  tags: juju-managed, devops-instance, devops-production
EOF
        juju deploy --repository "$MYDIR/charms" --config "$config_yaml" local:trusty/landscape-client
    fi

    #
    # deploy bootstrap-node charm
    #

    if ! juju status | grep -q bootstrap-node:; then
        juju deploy --repository $MYDIR/charms --to 0 local:trusty/bootstrap-node
        wait_deployed bootstrap-node
        subordinate_charms "bootstrap-node"
    fi

fi

#
# deploy langpack-o-matic charm
#

if juju status langpack-o-matic | grep -q 'agent-state:'; then
    echo 'WARNING: langpack-o-matic already deployed, skipping' >&2
else
    echo 'deploying langpack-o-matic'
    cat <<EOF >> "$config_yaml"
langpack-o-matic:
  gpg-public-key: |
$(echo "$GPG_PUB_KEY" | sed 's/^/    /')
  gpg-secret-key: |
$(echo "$GPG_SECRET_KEY" | sed 's/^/    /')
EOF
    juju deploy --repository "$MYDIR/charms" --constraints 'root-disk=40G' --config "$config_yaml" local:trusty/langpack-o-matic
    subordinate_charms "langpack-o-matic"
fi
